@@ -1851,6 +1851,14 @@ class TZTest(unittest.TestCase):
         with self.assertRaises(ValueError):
             tz.tzfile(BytesIO(b'BadFile'))
 
+    def testFilestreamWithNameRepr(self):
+        # If fileobj is a filestream with a "name" attribute this name should
+        # be reflected in the tz object's repr
+        fileobj = BytesIO(base64.b64decode(TZFILE_EST5EDT))
+        fileobj.name = 'foo'
+        tzc = tz.tzfile(fileobj)
+        self.assertEqual(repr(tzc), 'tzfile(' + repr('foo') + ')')
+
     def testRoundNonFullMinutes(self):
         # This timezone has an offset of 5992 seconds in 1900-01-01.
         tzc = tz.tzfile(BytesIO(base64.b64decode(EUROPE_HELSINKI)))
